default_settings = {
    "components": [],
    "middleware": [],
    "logging": {
        "setup": "selva.logging.setup.setup_logger",
        "root": "WARNING",
        "level": {},
        "enable": [],
        "disable": [],
    },
}
